#----------------------------------------------------------------------
# Copyright 1999-2003 Mitel Networks Corporation
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.
#----------------------------------------------------------------------

package esmith::NetworksDB;

require v5.6.0;
use strict;
use warnings;

use esmith::DB::db;
our @ISA = qw( esmith::DB::db );

=head1 NAME

esmith::NetworksDB - interface to esmith networks database

=head1 SYNOPSIS

    use esmith::NetworksDB;
    my $c = esmith::NetworksDB->open;

    # everything else works just like esmith::DB::db

=head1 DESCRIPTION

This module provides an abstracted interface to the esmith master
configuration database.

Unless otherwise noted, esmith::NetworksDB acts like esmith::DB::db.

=cut

=head2 open()

Like esmith::DB->open, but if given no $file it will try to open the
file in the ESMITH_NETWORKS_DB environment variable or
/home/e-smith/networks.

=begin testing

use_ok("esmith::NetworksDB");

$C = esmith::NetworksDB->open('10e-smith-lib/networks.conf');
isa_ok($C, 'esmith::NetworksDB');
is( $C->get("10.0.0.0")->prop('Mask'), "255.255.255.0", 
                                    "We can get stuff from the db");

=end testing

=cut

sub open
{
    my ( $class, $file ) = @_;
    $file = $file || $ENV{ESMITH_NETWORKS_DB} || "/home/e-smith/networks";
    return $class->SUPER::open($file);
}

=head2 open_ro()

Like esmith::DB->open_ro, but if given no $file it will try to open the
file in the ESMITH_NETWORKS_DB environment variable or
/home/e-smith/networks.

=begin testing

=end testing

=cut

sub open_ro
{
    my ( $class, $file ) = @_;
    $file = $file || $ENV{ESMITH_NETWORKS_DB} || "/home/e-smith/networks";
    return $class->SUPER::open_ro($file);
}

=head2 local_access_spec ([$access])

Compute the network/netmask entries which are to treated as local access.

There is also an optional access parameter which can further restrict 
the values returned. If C<access> is C<localhost>, this routine will only
return a single value, equating to access from localhost only.

If called in scalar context, the returned string is suitable for 
use in /etc/hosts.allow, smb.conf and httpd.conf, for example:

127.0.0.1 192.168.1.1/255.255.255.0

Note: The elements are space separated, which is suitable for use in
hosts.allow, smb.conf and httpd.conf. httpd.conf does not permit 
comma separated lists in C<allow from> directives.

If called in list context, returns the array of network/netmask strings.
It's trivial, of course, to convert an array to a comma separated list :-)

=cut

sub local_access_spec
{
    my $self   = shift;
    my $access = shift || "private";

    my @localAccess = ("127.0.0.1");

    if ( $access eq "localhost" )
    {
        # Nothing more to do
    }
    elsif ( $access eq "private" )
    {
        foreach my $network ( $self->get_all )
        {
            my $mask = $network->prop('Mask');
            push @localAccess, $network->key . "/$mask";
        }
    }
    elsif ( $access eq "public" )
    {
        @localAccess = ("ALL");
    }
    else
    {
        warn "local_access_spec: unknown access value $access\n";
    }
    return wantarray ? @localAccess : "@localAccess";
}

=head1 AUTHOR

SME Server Developers <bugs@e-smith.com>

=head1 SEE ALSO

L<esmith::DB::db>

L<esmith::DB::Record>

=cut

1;
